@@ -1,10 +1,9 @@
 #![cfg(feature = "reqwest")]
 
-use std::collections::HashMap;
 use std::sync::Arc;
 
-use authentic::credential::{HttpRealmCredentials, UsernamePasswordCredential};
-use authentic::reqwest::{BasicAuthentication, HttpAuthentication};
+use authentic::credential::UsernamePasswordCredential;
+use authentic::reqwest::BasicAuthentication;
 use authentic::{AuthenticationProtocol, AuthenticationStep, WithAuthentication};
 use http::StatusCode;
 
@@ -100,17 +99,22 @@ async fn test_basic_request() -> Result<(), Box<dyn std::error::Error + Send + S
 }
 
 /// Basic authentication passing the username and password in response to a 401 challenge.
+///
+/// `HttpAuthentication` is only supported with the `loop` feature.
+#[cfg(feature = "loop")]
 #[::tokio::test]
 async fn test_basic_challenge() -> Result<(), Box<dyn std::error::Error + Send + Sync + 'static>> {
     let client = reqwest::Client::new();
 
-    let mut realm_credentials = HashMap::new();
+    let mut realm_credentials = std::collections::HashMap::new();
     realm_credentials.insert(
         "Fake Realm".into(),
         Arc::new(UsernamePasswordCredential::new("username", "password")),
     );
-    let credential = Arc::new(HttpRealmCredentials::new(realm_credentials));
-    let mut authentication = HttpAuthentication::new(credential);
+    let credential = Arc::new(authentic::credential::HttpRealmCredentials::new(
+        realm_credentials,
+    ));
+    let mut authentication = authentic::reqwest::HttpAuthentication::new(credential);
 
     let mut status_codes = Vec::new();
 
